function [U, Db, B] = MWGS_UD(A, Dw)
%
% function [U, Db, B] = MWGS(A, Dw)
%
% Julia V. Tsyganova, Ulyanovsk State University
%
%  MWG-S Orthogonalization and Matrix Factorization
%  Bierman G. J. Factorization Methods For Discrete Sequential Estimation.
%  New York: Academic Press, 1977. P. 127 Theorem VI.4.1.
%
% A^T=UB^T, A Dw A^T=U Db U^T, B^T Dw B=Db
%
% INPUTS(with dimensions)
%      A   (m,n) The m x n rectangular matrix
%      Dw   (m)  The m x m diagonal matrix. It is stored as an m-vector
%
% OUTPUTS:
%      U  (n,n)  The n x n unit upper triangular factor (U) of the modified Cholesky
%              factors (U-D factors)
%      D  (n)    The n x n diagonal factor (D). It is stored as an n-vector.
%      B  (m,n)  The m x n transformation matrix
%
[m,n] = size(A); % get dimensions of matrix A
B    = A;        % initial value
U     = eye(n);  % initialize upper triangular part of U
% rows of A^T are to be orthononalized

for j=n-1:-1:1
    sigma = 0;
    for i=1:m
        sigma = sigma + B(i,j+1)^2 * Dw(i,i);
    end
    Db(j+1,j+1) = sigma;
    
    for k=1:j
        sigma = 0;
        for i=1:m
            sigma = sigma + B(i,k) * Dw(i,i) * B(i,j+1);
        end
        U(k,j+1) = sigma / Db(j+1,j+1);
        
        for i=1:m
            B(i,k) = B(i,k) - U(k,j+1) * B(i,j+1);
        end
    end
end

sigma = 0;

for i=1:m
    sigma = sigma + B(i,1)^2 * Dw(i,i);
end

Db(1,1) = sigma;

